'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var ResizeObserver = _interopDefault(require('resize-observer-polyfill'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var ApiContext = /*#__PURE__*/React.createContext({});

function enhancePropsWithClassname(props, newClass) {
  var _props$className = props.className,
      className = _props$className === void 0 ? '' : _props$className;
  return _extends({}, props, {
    className: className.length ? className.concat(" " + newClass) : newClass
  });
}

function createStyleTag(styles) {
  return React__default.createElement("style", {
    dangerouslySetInnerHTML: {
      __html: styles
    }
  });
}

function createLayoutClassname(type, api) {
  return "layout-block-" + type + "-" + Object.values(api).join('');
}

function createLayoutBlock(config) {
  var _config$api = config.api,
      apiFromDefaults = _config$api === void 0 ? {} : _config$api,
      name = config.name,
      styles = config.styles,
      template = config.template;

  function Block(props) {
    var apiFromContext = React.useContext(ApiContext);

    var _props$as = props.as,
        Tag = _props$as === void 0 ? 'div' : _props$as,
        children = props.children,
        rest = _objectWithoutPropertiesLoose(props, ["as", "children"]);

    var apiPrepopulated = _extends({}, apiFromDefaults, {}, apiFromContext[name]);

    var api = _extends({}, Object.keys(apiPrepopulated).reduce(function (apiFinal, key) {
      if (rest.hasOwnProperty(key)) {
        var _extends2;

        var apiValue = rest[key];
        delete rest[key];
        return _extends({}, apiFinal, (_extends2 = {}, _extends2[key] = apiValue, _extends2));
      }

      return apiFinal;
    }, apiPrepopulated));

    var styleClass = createLayoutClassname(name, api);
    var selector = Tag + "." + styleClass;
    return React__default.createElement(React__default.Fragment, null, createStyleTag(styles(selector, api)), template(Tag, enhancePropsWithClassname(rest, styleClass), children, api));
  }

  return Block;
}

var Box = /*#__PURE__*/createLayoutBlock({
  api: {
    padding: 'var(--s1)'
  },
  name: 'box',
  styles: function styles(selector, api) {
    var padding = api.padding;
    return "\n      " + selector + " {\n        padding: " + padding + ";\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), children);
  }
});

var Center = /*#__PURE__*/createLayoutBlock({
  api: {
    max: 'var(--measure)',
    isTextCentered: false,
    gutter: '0',
    isIntrinsic: false
  },
  name: 'center',
  styles: function styles(selector, api) {
    var max = api.max,
        isTextCentered = api.isTextCentered,
        gutter = api.gutter,
        isIntrinsic = api.isIntrinsic;
    return "\n      " + selector + " {\n        box-sizing: content-box;\n        margin-left: auto;\n        margin-right: auto;\n        max-width: " + max + ";\n        " + (isTextCentered ? 'text-align: center;' : '') + "\n        padding-left: " + gutter + ";\n        padding-right: " + gutter + ";\n        " + (isIntrinsic ? "\n          display:flex;\n          flex-direction: column;\n          align-items: center;\n        " : '') + "\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), children);
  }
});

var Cover = /*#__PURE__*/createLayoutBlock({
  api: {
    centered: 'h1',
    space: 'var(--s1)',
    minHeight: '100vh',
    isPadded: false
  },
  name: 'cover',
  styles: function styles(selector, api) {
    var centered = api.centered,
        space = api.space,
        minHeight = api.minHeight,
        isPadded = api.isPadded;
    return "\n      " + selector + " {\n        display: flex;\n        flex-direction: column;\n        min-height: " + minHeight + ";\n        padding: " + (isPadded ? space : '0') + ";\n      }\n\n      " + selector + " > * {\n        margin-top: " + space + ";\n        margin-bottom: " + space + ";\n      }\n\n      " + selector + " {\n        display: flex;\n        flex-direction: column;\n        min-height: " + minHeight + ";\n        padding: " + (isPadded ? space : '0') + ";\n      }\n\n      " + selector + " > :first-of-type:not(style):not(" + centered + ") {\n        margin-top: 0;\n      }\n\n      " + selector + " > :last-of-type:not(style):not(" + centered + ") {\n        margin-bottom: 0;\n      }\n\n      " + selector + " > " + centered + " {\n        margin-top: auto;\n        margin-bottom: auto;\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), children);
  }
});

var Frame = /*#__PURE__*/createLayoutBlock({
  api: {
    ratio: '16:9'
  },
  name: 'frame',
  styles: function styles(selector, api) {
    var ratio = api.ratio;

    var _ratio$split = ratio.split(':'),
        denominator = _ratio$split[0],
        numerator = _ratio$split[1];

    return "\n      " + selector + " {\n        --n: " + Number(numerator) + ";\n        --d: " + Number(denominator) + ";\n        padding-bottom: calc(var(--n) / var(--d) * 100%);\n        position: relative;\n      }\n\n      " + selector + " > * {\n        overflow: hidden;\n        position: absolute;\n        top: 0;\n        right: 0;\n        bottom: 0;\n        left: 0;\n        display: flex;\n        justify-content: center;\n        align-items: center;\n      }\n\n      " + selector + " > img,\n      " + selector + " > video {\n        width: 100%;\n        height: 100%;\n        object-fit: cover;\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), children);
  }
});

var FullBleed = /*#__PURE__*/createLayoutBlock({
  name: 'fullbleed',
  styles: function styles(selector) {
    return "\n      " + selector + " {\n        width: 100vw;\n        margin-left: 50%;\n        transform: translateX(-50%);\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), children);
  }
});

var useResizeObserver = function useResizeObserver() {
  var _useState = React.useState(null),
      entry = _useState[0],
      setEntry = _useState[1];

  var _useState2 = React.useState(null),
      node = _useState2[0],
      setNode = _useState2[1];

  var observer = React.useRef(null);
  var disconnect = React.useCallback(function () {
    var current = observer.current;
    current && current.disconnect();
  }, []);
  var observe = React.useCallback(function () {
    observer.current = new ResizeObserver(function (_ref) {
      var entry = _ref[0];
      return setEntry(entry);
    });
    node && observer.current.observe(node);
  }, [node]);
  React.useEffect(function () {
    observe();
    return function () {
      return disconnect();
    };
  }, [disconnect, observe]);
  return [setNode, entry];
};

var Grid = /*#__PURE__*/createLayoutBlock({
  api: {
    min: '250px',
    space: 'var(--s1)'
  },
  name: 'grid',
  styles: function styles(selector, api) {
    var min = api.min,
        space = api.space;
    return "\n      " + selector + " {\n        display: grid;\n        grid-gap: " + space + ";\n        grid-template-columns: 100%;\n      }\n\n      " + selector + ".aboveMin {\n        grid-template-columns: repeat(auto-fill, minmax(" + min + ", 1fr));\n      }\n\n      @supports (width: min(" + min + ", 100%)) {\n        " + selector + " {\n          grid-template-columns: repeat(auto-fit, minmax(min(" + min + ", 100%), 1fr));\n        }\n      }\n      ";
  },
  template: function template(Component, props, children, api) {
    var _useResizeObserver = useResizeObserver(),
        observerRef = _useResizeObserver[0],
        entry = _useResizeObserver[1];

    var childRef = React.useRef(null);
    var min = api.min;
    var isWide = false; // Add a test element to compare the min value to the container

    if (childRef.current && entry && entry.contentRect) {
      childRef.current.style.width = min;
      isWide = entry.contentRect.width > childRef.current.offsetWidth;
    }

    var containerClass = 'aboveMin';
    var className = props.className;

    var passedProps = _extends({}, props, {
      ref: observerRef,
      className: className.length ? isWide ? className.concat(" " + containerClass) : className : isWide ? containerClass : className
    });

    return React__default.createElement(Component, Object.assign({}, passedProps), children, React__default.createElement("div", {
      ref: childRef
    }));
  }
});

var Group = /*#__PURE__*/createLayoutBlock({
  api: {
    align: 'center',
    justify: 'flex-start',
    space: 'var(--s1)'
  },
  name: 'group',
  styles: function styles(selector, api) {
    var align = api.align,
        justify = api.justify,
        space = api.space;
    return "\n      " + selector + " {\n        overflow: hidden;\n      }\n\n      " + selector + " > * {\n        display: flex;\n        flex-wrap: wrap;\n        justify-content: " + justify + ";\n        align-items: " + align + ";\n        margin: calc(" + space + " / 2 * -1);\n      }\n\n      " + selector + " > * > * {\n        margin: calc(" + space + " / 2);\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), React__default.createElement("div", null, children));
  }
});

var Imposter = /*#__PURE__*/createLayoutBlock({
  api: {
    isBreakout: false,
    margin: '0px',
    isFixed: false
  },
  name: 'imposter',
  styles: function styles(selector, api) {
    var isBreakout = api.isBreakout,
        margin = api.margin,
        isFixed = api.isFixed;
    return "\n      " + selector + " {\n        position: absolute;\n        top: 50%;\n        left: 50%;\n        transform: translate(-50%, -50%);\n        " + (!isBreakout ? "\n          max-width: calc(100% - (" + margin + " * 2));\n          max-height: calc(100% - (" + margin + " * 2));\n          overflow: auto;" : '') + "\n        " + (isFixed ? "\n          position: fixed;" : '') + "\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), children);
  }
});

var Sidebar = /*#__PURE__*/createLayoutBlock({
  api: {
    side: 'left',
    sideWidth: null,
    contentMin: '50%',
    space: 'var(--s1)',
    isStretch: false
  },
  name: 'sidebar',
  styles: function styles(selector, api) {
    var side = api.side,
        sideWidth = api.sideWidth,
        contentMin = api.contentMin,
        space = api.space,
        isStretch = api.isStretch;
    return "\n      " + selector + " {\n        overflow: hidden;\n      }\n\n      " + selector + " > * {\n        display: flex;\n        flex-wrap: wrap;\n        " + (!isStretch ? "align-items: flex-start;" : '') + "\n        margin: calc(" + space + " / 2 * -1);\n      }\n\n      " + selector + " > * > * {\n        margin: calc(" + space + " / 2);\n        " + (sideWidth ? "flex-basis: " + sideWidth + ";" : '') + "\n        flex-grow: 1;\n      }\n\n      " + selector + " > * > :" + (side === 'left' ? 'last-of-type' : 'first-of-type') + ":not(style) {\n        flex-basis: 0;\n        flex-grow: 999;\n        min-width: calc(" + contentMin + " - " + space + ");\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), React__default.createElement("div", null, children));
  }
});

var Stack = /*#__PURE__*/createLayoutBlock({
  api: {
    recursive: false,
    splitAfter: null,
    space: 'var(--s1)'
  },
  name: 'stack',
  styles: function styles(selector, api) {
    var recursive = api.recursive,
        space = api.space,
        splitAfter = api.splitAfter;
    return "\n    " + selector + " {\n      display: flex;\n      flex-direction: column;\n      justify-content: flex-start;\n    }\n\n    " + selector + " " + (!recursive ? '>' : '') + " *:not(style) ~ *:not(style) {\n      margin-top: " + space + ";\n    }\n  \n    " + (splitAfter ? "\n        " + selector + ":only-child {\n        height: 100%;\n      }\n      " + selector + " > :nth-child(" + splitAfter + ") {\n        margin-bottom: auto;\n      }\n    " : '') + "\n  ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), children);
  }
});

var Switcher = /*#__PURE__*/createLayoutBlock({
  api: {
    threshold: 'var(--measure)',
    space: 'var(--s1)',
    limit: 4
  },
  name: 'switcher',
  styles: function styles(selector, api) {
    var threshold = api.threshold,
        space = api.space,
        limit = api.limit;
    return "\n      " + selector + " > * {\n        display: flex;\n        flex-wrap: wrap;\n        margin: calc((" + space + " / 2) * -1);\n      }\n\n      " + selector + " > * > * {\n        flex-grow: 1;\n        flex-basis: calc((" + threshold + " - (100% - " + space + ")) * 999);\n        margin: calc(" + space + " / 2);\n      }\n\n      " + selector + " > * > :nth-last-of-type(n+" + (Number(limit) + 1) + "):not(style),\n      " + selector + " > * > :nth-last-of-type(n+" + (Number(limit) + 1) + "):not(style) ~ *:not(style) {\n        flex-basis: 100%;\n      }\n      ";
  },
  template: function template(Component, props, children) {
    return React__default.createElement(Component, Object.assign({}, props), React__default.createElement("div", null, children));
  }
});

exports.Box = Box;
exports.Center = Center;
exports.Cover = Cover;
exports.Frame = Frame;
exports.FullBleed = FullBleed;
exports.Grid = Grid;
exports.Group = Group;
exports.Imposter = Imposter;
exports.Sidebar = Sidebar;
exports.Stack = Stack;
exports.Switcher = Switcher;
//# sourceMappingURL=layout-blocks.cjs.development.js.map
